function xhr(url, callback){
    var xhr = new XMLHttpRequest();
    xhr.onreadystatechange = function(){
        if(this.readyState === 4){
            if(this.status === 200){
                callback(null, this.responseText);
            }else{
                callback(new Error(this.status));
            }
        }
    };
    xhr.open("GET", url);
    xhr.send("\r\n");
}

function CurrencyMarket(marketId, baseCode, assetCode){
    this._marketId = marketId;
    this._baseCode = baseCode;
    this._assetCode = assetCode;
}

Object.defineProperties(CurrencyMarket.prototype, {
    marketId: {
        get: function(){
            return this._marketId;
        }
    },
    baseCode: {
        get: function(){
            return this._baseCode;
        }
    },
    assetCode: {
        get: function(){
            return this._assetCode;
        }
    }
});

function MarketSummary(marketId, data){
    this._marketId = marketId;
    this._data = data;
}

Object.defineProperties(MarketSummary.prototype, {
    marketId: {
        get: function(){
            return this._marketId;
        }
    }
});

var CoinExchangeAPI = {
    endpoint: "https://www.coinexchange.io/api",
    getMarkets: function(callback){
        xhr(this.endpoint + "/v1/getmarkets", function(error, data){
            if(error){
                callback(error);
            }else{
                var json = JSON.parse(data);
                callback(null, json.result.map(function(data){
                    return new CurrencyMarket(data.MarketID, data.BaseCurrencyCode, data.MarketAssetCode);
                }));
            }
        });
    },
    getMarketSummaries: function(callback){
        xhr(this.endpoint + "/v1/getmarketsummaries", function(error, data){
            if(error){
                callback(error);
            }else{
                var json = JSON.parse(data);
                callback(null, json.result.map(function(data){
                    return new MarketSummary(data.MarketID, data);
                }));
            }
        });
    },
    getOrderBook: function(marketId, callback){
        xhr(this.endpoint + "/v1/getorderbook?market_id=" + marketId, function(error, data){
            if(error){
                callback(error);
            }else{
                var json = JSON.parse(data);
                callback(null, json.result);
            }
        });
    },
    connectMarket: function(marketId, callback){
        var md_ws = new WebSocket("wss://wss.coinexchange.io:3001/marketdata");
        md_ws.onopen = function(){
            console.log("open");
            md_ws.send(JSON.stringify({
                type: "join_channel",
                market_id: marketId,
                ws_auth_token: ""
            }));
        };
        md_ws.onclose = function(){
            console.log('close');
        };
        md_ws.onmessage = function(event){
            var data = JSON.parse(event.data);
            callback(null, data);
        }
        return md_ws;
    }
};

function ExchangeMarketDatabase(){
    this._marketTable = {};
    this._summaryTable = {};
    this._listeners = {};
}

ExchangeMarketDatabase.prototype.updateMarkets = function(markets){
    for(var i = 0, l = markets.length; i < l; i++){
        var market = markets[i];
        this._marketTable[market.marketId] = market;
    }
};

ExchangeMarketDatabase.prototype.marketsForAssetCode = function(code){
    var list = [];
    for(var i in this._marketTable){
        if(!this._marketTable.hasOwnProperty(i)) continue;
        var market = this._marketTable[i];
        if(market.assetCode === code){
            list.push(market)
        }
    }
    return list;
};

ExchangeMarketDatabase.prototype.updateMarketSummaries = function(summaries){
    for(var i = 0, l = summaries.length; i < l; i++){
        var summary = summaries[i];
        this._summaryTable[summary.marketId] = summary;
    }
};

ExchangeMarketDatabase.prototype.summariesForMarketIDs = function(marketIds){
    var self = this;
    return marketIds.map(function(marketId){
        return self._summaryTable[marketId];
    });
};

function renderGauge(context, x, y, width, height, value, gaugeVolume){
    var colors = [
        "#00ff00",
        "#0000ff",
        "#ff00ff",
        "#00ffff"
    ];
    context.fillStyle = "#ffffff";
    context.fillRect(x, y, width, 10);

    var frontColor = "";
    var backColor = "";
    var n = Math.floor(value / gaugeVolume);
    var m = value - n * gaugeVolume;
    if(n >= colors.length){
        var nn = (n - colors.length) % 2;
        frontColor = colors[colors.length - (2 - nn)];
        backColor = colors[colors.length - 1 - nn];
    }else{
        if(n === 0){
            backColor = "#000000";
            if(m < gaugeVolume / 4){
                frontColor = "#ff0000";
            }else if(m < gaugeVolume / 2){
                frontColor = "#ffff00";
            }else{
                frontColor = colors[0];
            }
        }else{
            backColor = colors[n - 1];
            frontColor = colors[n];
        }
    }
    context.fillStyle = backColor;
    context.fillRect(x + 2, y + 2, width - 4, height - 4);
    context.fillStyle = frontColor;
    var gauge = Math.floor((width - 4) * m / gaugeVolume);
    if(gauge < 1) gauge = 1;
    context.fillRect(x + 2, y + 2, gauge, height - 4);
}

function renderEnemy(context, image, enemy, x, y, w){
    var gaugeVolume = enemy.gauge;
    context.mozImageSmoothingEnabled = false;
    context.webkitImageSmoothingEnabled = false;
    context.msImageSmoothingEnabled = false;
    context.imageSmoothingEnabled = false;
    if(enemy.damageTime > 0 && Math.floor(enemy.damageTime-- / 8) % 2 === 0){
    }else{
        context.drawImage(image, x, y, w, w);
    }
    context.mozImageSmoothingEnabled = true;
    context.webkitImageSmoothingEnabled = true;
    context.msImageSmoothingEnabled = true;
    context.imageSmoothingEnabled = true;

    renderGauge(context, x + 10, y + 4, w - 20 - 30, 10, parseFloat(enemy.hp) * parseFloat(enemy.level), gaugeVolume);
    context.font = "16px sans-serif";
    context.fillStyle = "#ffffff";
    var value = enemy.hp / gaugeVolume * parseFloat(enemy.level) * 100;
    if(isNaN(value)){
        console.log("NaN", enemy, enemy.hp, gaugeVolume, enemy.level);
        throw new Error();
    }
    context.fillText((enemy.hp / gaugeVolume * parseFloat(enemy.level) * 100).toFixed(1) + "%", x + w - 30, y + 14);

    context.font = "20px sans-serif";
    context.fillStyle = "#00c0ff";
    context.fillText(enemy.level + "", x + 8, y + 16 + w);
}

function EnemyManager(){
    this._enemies = [];
}

function formatLevel(level){
    return "" + parseFloat(level).toFixed(8);
}

EnemyManager.prototype.add = function(level, hp, time, gauge){
    level = formatLevel(level);
    if(!this._enemies[level]){
        this._enemies[level] = {
            level: level,
            hp: 0,
            gauge: gauge
        };
    }
    this._enemies[level].hp += hp;
    var deads = [];
    for(var i in this._enemies){
        if(!this._enemies.hasOwnProperty(i)) continue;
        if(this._enemies[i].dead){
            deads.push(i);
        }
    }
    for(var i = 0, l = deads.length; i < l; i++){
        delete this._enemies[deads[i]];
    }
};

EnemyManager.prototype.damage = function(level, damage){
    level = formatLevel(level);
    if(!this._enemies[level]){
        throw new Error("does not exist: " + level);
    }
    this._enemies[level].hp -= damage;
    this._enemies[level].damageTime = 96;
    if(this._enemies[level].hp < 0.000000001){
        this._enemies[level].dead = true;
    }
    this.onDamage(level, damage);
};

EnemyManager.prototype.getEnemies = function(){
    var enemies = [];
    for(var i in this._enemies){
        if(!this._enemies.hasOwnProperty(i)) continue;
        enemies.push(this._enemies[i])
    }
    var sorted = enemies.sort(function(a, b){
        if(a.level < b.level) return -1;
        if(a.level > b.level) return 1;
        return 0;
    });
    return sorted;
};

function BattleVisualizer(canvas){
    this._canvas = canvas;
    this._context = canvas.getContext("2d");
    this._enemyManager = new EnemyManager();
    var self = this;
    this._enemyManager.onDamage = function(level, damage){
        self.attack = 8;
    };
    this._bossManager = new EnemyManager();
    this._bossManager.onDamage = function(level, damage){
        self.attack = 8;
    };
    this.IMG_DOGE = document.getElementById("img_doge");
    this.IMG_XP = document.getElementById("img_xp");
    this.IMG_SATOSHI = document.getElementById("img_satoshi");
    this.attack = 0;
}

Object.defineProperties(BattleVisualizer.prototype, {
    enemyManager: {
        get: function(){
            return this._enemyManager;
        }
    },
    bossManager: {
        get: function(){
            return this._bossManager;
        }
    }
});

BattleVisualizer.prototype.render = function(){
    var scaleW = Math.floor(Math.floor(this._canvas.width / 4) / 6);
    var scaleH = Math.floor(Math.floor(this._canvas.height / 4) / 3);
    var scale = Math.min(scaleW, scaleH);
    var w = scale * 4;
    var offsetX = (this._canvas.width - (w + w / 4) * 5) / 2;
    var offsetY = (this._canvas.height - (w + w / 4) * 2) / 2;
    var context = this._context;
    context.clearRect(0, 0, this._canvas.width, this._canvas.height);
    context.mozImageSmoothingEnabled = false;
    context.webkitImageSmoothingEnabled = false;
    context.msImageSmoothingEnabled = false;
    context.imageSmoothingEnabled = false;
    if(this.attack > 0){
        context.drawImage(this.IMG_XP, offsetX + w * 5 - scale * 2, offsetY + w / 2 + 20, w, w);
        this.attack--;
    }else{
        context.drawImage(this.IMG_XP, offsetX + w * 5, offsetY + w / 2 + 20, w, w);
    }
    context.mozImageSmoothingEnabled = true;
    context.webkitImageSmoothingEnabled = true;
    context.msImageSmoothingEnabled = true;
    context.imageSmoothingEnabled = true;
    var enemies = this._enemyManager.getEnemies();
    for(var i = 0, l = enemies.length; i < l && i < 4; i++){
        var enemy = enemies[i];
        if(!enemy.dead){
            renderEnemy(context, this.IMG_DOGE, enemy, offsetX + (w + w / 4) * (1 - Math.floor(i / 2)) + 20 + w, offsetY + (i % 2) * (w + w / 4) + 20, w);
        }
    }
    var bosses = this._bossManager.getEnemies();
    for(var i = 0, l = bosses.length; i < l && i < 1; i++){
        var boss = bosses[i];
        renderEnemy(context, this.IMG_SATOSHI, boss, offsetX, offsetY + w / 2 + 20, w);
    }
    var self = this;
    requestAnimationFrame(function(){
        self.render();
    });
};

var canvas;
window.addEventListener('load', function(){
    canvas = document.getElementById("canvas");
    canvas.width = window.innerWidth;
    canvas.height = window.innerHeight;
    var visualizer = new BattleVisualizer(canvas);
    visualizer.render();
    var db = new ExchangeMarketDatabase();
    CoinExchangeAPI.getMarkets(function(error, markets){
        if(error){
            console.log(error);
            return;
        }
        db.updateMarkets(markets);
        CoinExchangeAPI.getMarketSummaries(function(error, summaries){
            if(error){
                console.log(error);
                return;
            }
            db.updateMarketSummaries(summaries);
            var markets = db.marketsForAssetCode("XP");
            var summaries = db.summariesForMarketIDs(markets.map(function(market){
                return market.marketId;
            }));
            function observeMarket(manager, marketId, gauge){
                CoinExchangeAPI.getOrderBook(marketId, function(error, data){
                    for(var i = 0, l = data.SellOrders.length; i < l; i++){
                        var order = data.SellOrders[i];
                        manager.add(order.Price, parseFloat(order.Quantity), order.OrderTime, gauge);
                    }
                    CoinExchangeAPI.connectMarket(marketId, function(error, data){
                        if(data.type === "recent_market_trade"){
                        }else if(data.type === "update_sell_order"){
                            if(data.direction === "remove"){
                                manager.damage(data.price, parseFloat(data.quantity));
                            }else if(data.direction === "add"){
                                manager.add(data.price, parseFloat(data.quantity), order.OrderTime, gauge);
                            }
                        }
                    });
                });
            }
            console.log(summaries);

            observeMarket(visualizer.enemyManager, summaries[1].marketId, summaries[1]._data.Volume / 24 / 4);
            observeMarket(visualizer.bossManager, summaries[0].marketId, summaries[0]._data.Volume / 24 / 4);
        });
    });
});

window.addEventListener('resize', function(){
    canvas.width = window.innerWidth;
    canvas.height = window.innerHeight;
});
